# import os

# from azureml.core.environment import Environment
# from azureml.core.model import InferenceConfig, Model
# from azureml.core.webservice import AciWebservice, Webservice

# from azure_helper.utils.aml_interface import AMLInterface

# __here__ = os.path.dirname(__file__)


# def get_inference_config(aml_interface):
#     aml_env = Environment.get(
#         workspace=aml_interface.workspace,
#         name=AML_ENV_NAME,
#     )
#     scoring_script_path = os.path.join(__here__, "score.py")
#     inference_config = InferenceConfig(
#         entry_script=scoring_script_path,
#         environment=aml_env,
#     )
#     return inference_config


# def deploy_service(aml_interface):
#     inference_config = get_inference_config(aml_interface)
#     deployment_config = AciWebservice.deploy_configuration(
#         cpu_cores=1,
#         memory_gb=1,
#     )
#     model = aml_interface.workspace.models.get(MODEL_NAME)
#     service = Model.deploy(
#         aml_interface.workspace,
#         DEPLOYMENT_SERVICE_NAME,
#         [model],
#         inference_config,
#         deployment_config,
#     )
#     service.wait_for_deployment(show_output=True)
#     print(service.scoring_uri)


# def update_service(aml_interface):
#     inference_config = get_inference_config(aml_interface)
#     service = Webservice(
#         name=DEPLOYMENT_SERVICE_NAME,
#         workspace=aml_interface.workspace,
#     )
#     model = aml_interface.workspace.models.get(MODEL_NAME)
#     service.update(models=[model], inference_config=inference_config)
#     print(service.state)
#     print(service.scoring_uri)


# def main():
#     # Retrieve vars from env
#     workspace_name = os.environ["AML_WORKSPACE_NAME"]
#     resource_group = os.environ["RESOURCE_GROUP"]
#     subscription_id = os.environ["SUBSCRIPTION_ID"]

#     spn_credentials = {
#         "tenant_id": os.environ["TENANT_ID"],
#         "service_principal_id": os.environ["SPN_ID"],
#         "service_principal_password": os.environ["SPN_PASSWORD"],
#     }

#     aml_interface = AMLInterface(
#         spn_credentials,
#         subscription_id,
#         workspace_name,
#         resource_group,
#     )
#     webservices = aml_interface.workspace.webservices.keys()
#     if DEPLOYMENT_SERVICE_NAME not in webservices:
#         deploy_service(aml_interface)
#     else:
#         update_service(aml_interface)



from unittest import TestCase
from unittest.mock import Mock, patch

from azureml.core import ScriptRunConfig

from azure_helper.steps.deploy_aml_model import (
    deploy_service,
    get_inference_config,
    update_service,
)


@patch("src.my_custom_package.deploy_aml_model.InferenceConfig")
@patch("src.my_custom_package.deploy_aml_model.Environment")
def test_get_inference_config(mock_environment, mock_config):
    # We are not testing Azure ML SDK functionality in a unit test
    # Rather that the correct calls to the correct objects are made
    # using mock objects
    mock_aml_interface = Mock()
    get_inference_config(mock_aml_interface)

    mock_environment.get.assert_called_once()
    mock_config.assert_called_once()


@patch("src.my_custom_package.deploy_aml_model.Model")
@patch("src.my_custom_package.deploy_aml_model.AciWebservice")
@patch("src.my_custom_package.deploy_aml_model.get_inference_config")
def test_deploy_service(
    mock_get_inference_config,
    mock_AciWebservice,
    mock_Model,
):
    mock_aml_interface = Mock()
    mock_service = Mock()
    mock_service.scoring_uri = "https://foo.bar/"
    mock_Model.deploy.return_value = mock_service

    deploy_service(mock_aml_interface)
    # mock_get_inference_config is tested independently above
    mock_get_inference_config.assert_called_once()

    mock_AciWebservice.deploy_configuration.assert_called_once_with(
        cpu_cores=1,
        memory_gb=1,
    )
    mock_service.wait_for_deployment.assert_called_once()


@patch("src.my_custom_package.deploy_aml_model.Webservice")
@patch("src.my_custom_package.deploy_aml_model.get_inference_config")
def test_update_service(mock_get_inference_config, mock_Webservice):
    mock_aml_interface = Mock()
    mock_get_inference_config.return_value = "test_inference_config"
    mock_aml_interface.workspace.models.get.return_value = "test_model_name"
    mock_service = Mock()
    mock_Webservice.return_value = mock_service

    update_service(mock_aml_interface)
    # mock_get_inference_config is tested independently above
    mock_get_inference_config.assert_called_once()
    mock_service.update.assert_called_once_with(
        models=["test_model_name"],
        inference_config="test_inference_config",
    )



from unittest import TestCase

import pandas as pd

from azure_helper.utils.transform_data import remove_collinear_cols


class TestTransformData(TestCase):
    def test_remove_collinear_cols(self):
        test_data_column_names = [
            "A",
            "B",
            "C",
            "D",
            "E",
            "F",
            "G",
            "H",
            "I",
            "J",
        ]
        test_X_data = pd.DataFrame(
            [
                [
                    -0.25,
                    -0.90,
                    0.11,
                    -0.12,
                    -0.04,
                    0.21,
                    -0.41,
                    -0.69,
                    -0.13,
                    -0.06,
                ],
                [
                    0.79,
                    0.13,
                    -0.1,
                    0.87,
                    -2.2,
                    0.4,
                    1.38,
                    -0.78,
                    -1.98,
                    0.5,
                ],
            ],
            columns=test_data_column_names,
        )

        self.assertIn("D", test_X_data.columns)
        self.assertIn("I", test_X_data.columns)

        X_data_transformed = remove_collinear_cols(test_X_data)

        self.assertIsInstance(X_data_transformed, pd.DataFrame)
        self.assertNotIn("D", X_data_transformed.columns)
        self.assertNotIn("I", X_data_transformed.columns)
        self.assertIn("A", X_data_transformed.columns)
        self.assertIn("E", X_data_transformed.columns)
        self.assertIn("J", X_data_transformed.columns)
        self.assertEqual(X_data_transformed.loc[0, "C"], 0.11)
        self.assertEqual(X_data_transformed.loc[1, "E"], -2.2)



# # pylint: skip-file
# import json

# import joblib
# import numpy as np
# from azureml.core.model import Model

# from azure_helper.utils.const import MODEL_NAME


# def init():
#     global model
#     model_path = Model.get_model_path(MODEL_NAME)
#     model = joblib.load(model_path)


# def run(data):
#     try:
#         data = json.loads(data)
#         data = data["data"]
#         result = model.predict(np.array(data))
#         return result.tolist()
#     except Exception as e:
#         error = str(e)
#         return error



import os
from pathlib import Path

from azureml.core import Environment, Experiment, ScriptRunConfig
from azureml.core.runconfig import DockerConfiguration

from azure_helper.logger import get_logger
from azure_helper.utils.aml_interface import AMLInterface

log = get_logger()


class AMLExperiment:
    def __init__(
        self,
        aml_interface: AMLInterface,
        aml_compute_name: str,
        aml_compute_instance: str,
        env_name: str,
        experiment_name: str,
        clean_after_run: bool = True,
    ) -> None:
        self.interface = aml_interface
        self.aml_compute_name = aml_compute_name
        self.aml_compute_instance = aml_compute_instance
        self.env_name = env_name
        self.experiment_name = experiment_name
        self.clean_after_run = clean_after_run

    def submit_run(self):

        experiment = Experiment(self.interface.workspace, self.experiment_name)
        # src_dir = __here__
        src_dir = str(Path.cwd())

        docker_config = DockerConfiguration(use_docker=True)
        run_config = ScriptRunConfig(
            source_directory=src_dir,
            script="azure_helper/steps/train.py",
            docker_runtime_config=docker_config,
        )

        compute_target = self.interface.get_compute_target(
            self.aml_compute_name,
            self.aml_compute_instance,
        )

        run_config.run_config.target = compute_target

        aml_run_env = Environment.get(
            self.interface.workspace,
            self.env_name,
        )
        run_config.run_config.environment = aml_run_env

        log.info("Submitting Run")
        run = experiment.submit(config=run_config)
        run.wait_for_completion(show_output=True)
        log.info(f"Run completed : {run.get_metrics()}")

        if self.clean_after_run:
            log.info("Deleting compute instance.")
            compute_target.delete()



import pandas as pd

from azure_helper.utils.aml_interface import AMLInterface
from azure_helper.utils.blob_storage_interface import BlobStorageInterface


class CreateData:
    def __init__(
        self,
        project_name: str,
        train_container: str = "train",
        test_container: str = "test",
    ):
        self.project_name = project_name
        self.train_container = train_container
        self.test_container = test_container

    def upload_training_data(
        self,
        blob_storage_interface: BlobStorageInterface,
        x_train: pd.DataFrame,
        y_train: pd.DataFrame,
    ):
        blob_storage_interface.upload_df_to_blob(
            dataframe=x_train,
            container_name=f"{self.project_name}",
            blob_path=f"{self.train_container}/X_train.csv",
        )
        blob_storage_interface.upload_df_to_blob(
            dataframe=y_train,
            container_name=f"{self.project_name}",
            blob_path=f"{self.train_container}/y_train.csv",
        )

    def upload_validation_data(
        self,
        blob_storage_interface: BlobStorageInterface,
        x_valid: pd.DataFrame,
        y_valid: pd.DataFrame,
    ):
        # Data to be used during model validation
        blob_storage_interface.upload_df_to_blob(
            dataframe=x_valid,
            container_name=f"{self.project_name}",
            blob_path=f"{self.train_container}/X_valid.csv",
        )
        blob_storage_interface.upload_df_to_blob(
            dataframe=y_valid,
            container_name=f"{self.project_name}",
            blob_path=f"{self.train_container}/y_valid.csv",
        )

    def upload_test_data(
        self,
        blob_storage_interface: BlobStorageInterface,
        x_test: pd.DataFrame,
        y_test: pd.DataFrame,
    ):
        # Data to be used during model evaluation
        # So stored in the training container
        blob_storage_interface.upload_df_to_blob(
            dataframe=x_test,
            container_name=f"{self.project_name}",
            blob_path=f"{self.test_container}/X_test.csv",
        )
        blob_storage_interface.upload_df_to_blob(
            dataframe=y_test,
            container_name=f"{self.project_name}",
            blob_path=f"{self.test_container}/y_test.csv",
        )



from pathlib import Path
from unittest.mock import Mock

import pytest
from azureml.core.conda_dependencies import CondaDependencies

from azure_helper.logger import get_logger
from azure_helper.steps.create_aml_env import AMLEnvironment, EnvSpecs

log = get_logger()
test_module = "azure_helper.steps.create_aml_env"

test_whl_dir = Path(__file__).parents[1] / Path("dist")


# def test_get_dist_dir(caplog, mocker):

#     mock_get_dist_dir = mocker.patch(
#         f"{test_module}.get_dist_dir",
#     )
#     mock_get_dist_dir.side_effect = [test_whl_dir, FileNotFoundError]

#     dist_dir = mock_get_dist_dir()

#     assert isinstance(dist_dir, Path)
#     assert dist_dir.stem == "dist"

#     with pytest.raises(FileNotFoundError):
#         dist_dir = mock_get_dist_dir()

#         assert len(caplog) == 1
#         assert (
#             f"Couldn't find distribution directory {test_whl_dir}"
#             in caplog.records[1].message
#         )


class TestRetrieveWhlFilepath:
    def test_validate_dir(self, mocker, tmp_path, caplog):
        dist_dir = tmp_path / Path("test")
        dist_dir.mkdir()

        aml_env = AMLEnvironment(dist_dir=dist_dir)

        aml_env.validate_dir()

        assert len(caplog.records) == 1
        assert f"Looking for wheel file in {dist_dir}." in caplog.records[0].message

        mock_dir = mocker.patch(f"{test_module}.Path.is_dir")
        mock_dir.side_effect = [False]
        with pytest.raises(FileNotFoundError):
            aml_env.validate_dir()

    def test_retrieve_whl_filepath_fails_no_dist_dir(self, mocker, tmp_path, caplog):

        dist_dir = tmp_path / Path("test")

        mock_dir = mocker.patch(f"{test_module}.Path.is_dir")
        mock_dir.side_effect = [False]
        aml_env = AMLEnvironment(dist_dir=dist_dir)

        with pytest.raises(FileNotFoundError):
            aml_env.retrieve_whl_filepath()

        assert len(caplog.records) == 1
        assert (
            f"Couldn't find distribution directory {dist_dir}"
            in caplog.records[0].message
        )

    def test_retrieve_whl_filepath_fails_no_file(self, tmp_path, caplog):

        file = "azure_helper-0.1-py3.tar.gz"
        whl_filepath = tmp_path / Path(file)
        whl_filepath.touch()

        aml_env = AMLEnvironment(dist_dir=tmp_path)
        with pytest.raises(FileNotFoundError):
            wheel = aml_env.retrieve_whl_filepath()

        assert len(caplog.records) == 2
        assert "Couldn't find wheel distribution" in caplog.records[1].message

    def test_retrieve_whl_filepath(self, tmp_path, caplog):

        file = "azure_helper-0.1-py3.whl"
        whl_filepath = tmp_path / Path(file)
        whl_filepath.touch()

        aml_env = AMLEnvironment(dist_dir=tmp_path)

        filepath = aml_env.retrieve_whl_filepath()

        assert isinstance(filepath, Path)
        assert filepath.suffix == ".whl"
        assert filepath.stem == "azure_helper-0.1-py3"

        assert len(caplog.records) == 2
        assert f"Looking for wheel file in {tmp_path}." in caplog.records[0].message

    def test_create_aml_environment_with_pip(self, mocker, tmp_path):

        # Initiate the class
        env = AMLEnvironment(dist_dir=tmp_path)

        # Mock the Environment class that is called inside AMLEnv
        mock_env = mocker.patch(f"{test_module}.Environment")

        # Mock the AMLInterface class that is called inside AMLEnv
        mock_aml_interface = mocker.patch(f"{test_module}.AMLInterface")

        # Create some temp files and dirs to do the tests
        file = "azure_helper-0.1-py3.whl"
        whl_filepath = tmp_path / Path(file)
        whl_filepath.touch()

        spec_dir = tmp_path / Path("specs")
        spec_dir.mkdir()

        spec_file = spec_dir / Path("requirements.txt")
        spec_file.write_text("numpy==1.18.2")
        spec_file.touch()

        # Define the type of env we want to test, ie pip environment type
        env_specs = EnvSpecs(flavor="pip", spec_file=spec_file)

        aml_env = env.create_aml_environment(
            env_name="test_env_name",
            env_specs=env_specs,
            aml_interface=mock_aml_interface,
        )

        mock_env.from_pip_requirements.assert_called_once_with(
            name="test_env_name",
            file_path=str(spec_file),
        )

        aml_env.add_private_pip_wheel.assert_called_once_with(
            workspace=mock_aml_interface.workspace,
            file_path=whl_filepath,
            exist_ok=True,
        )

    def test_create_aml_environment_with_docker(self, mocker, tmp_path):

        # Initiate the class
        env = AMLEnvironment(dist_dir=tmp_path)

        # Mock the Environment class that is called inside AMLEnv
        mock_env = mocker.patch(f"{test_module}.Environment")
        mock_env_obj = Mock()
        mock_env.return_value = mock_env_obj

        # Mock the AMLInterface class that is called inside AMLEnv
        mock_aml_interface = mocker.patch(f"{test_module}.AMLInterface")

        # Create some temp files and dirs to do the tests
        file = "azure_helper-0.1-py3.whl"
        whl_filepath = tmp_path / Path(file)
        whl_filepath.touch()

        spec_dir = tmp_path / Path("specs")
        spec_dir.mkdir()

        spec_file = spec_dir / Path("Dockerfile")
        spec_file.write_text("FROM ubuntu:20.04")
        spec_file.touch()

        # Define the type of env we want to test, ie pip environment type
        env_specs = EnvSpecs(flavor="docker", spec_file=spec_file)

        aml_env = env.create_aml_environment(
            env_name="test_env_name",
            env_specs=env_specs,
            aml_interface=mock_aml_interface,
        )

        mock_env.from_dockerfile.assert_called_once_with(
            name="test_env_name",
            dockerfile=str(spec_file),
        )

        aml_env.add_private_pip_wheel.assert_called_once_with(
            workspace=mock_aml_interface.workspace,
            file_path=whl_filepath,
            exist_ok=True,
        )

        log.warning(f"{mock_env_obj.python.conda_dependencies.pip_packages}")
        # pkg_list = [pkg for pkg in mock_env_obj.python.conda_dependencies.pip_packages]
        # assert "numpy==1.18.2" in pkg_list

        # assert aml_env.docker.base_image == "python:3.8.13"

    def test_create_aml_environment_with_error(self, mocker, tmp_path, caplog):

        # Initiate the class
        env = AMLEnvironment(dist_dir=tmp_path)

        # Mock the Environment class that is called inside AMLEnv
        mock_env = mocker.patch(f"{test_module}.Environment")

        # Mock the AMLInterface class that is called inside AMLEnv
        mock_aml_interface = mocker.patch(f"{test_module}.AMLInterface")

        # Create some temp files and dirs to do the tests
        file = "azure_helper-0.1-py3.whl"
        whl_filepath = tmp_path / Path(file)
        whl_filepath.touch()

        spec_dir = tmp_path / Path("specs")
        spec_dir.mkdir()

        spec_file = spec_dir / Path("Dockerfile")
        spec_file.write_text("FROM ubuntu:20.04")
        spec_file.touch()

        # Define the type of env we want to test, ie pip environment type
        env_specs = EnvSpecs(flavor="foobar", spec_file=spec_file)

        with pytest.raises(ValueError):

            aml_env = env.create_aml_environment(
                env_name="test_env_name",
                env_specs=env_specs,
                aml_interface=mock_aml_interface,
            )

        assert len(caplog.records) == 1
        assert (
            "env_specs flavor foobar is not a valid one. Only 'pip', 'conda', or 'docker' are valide choices."
            in caplog.records[0].message
        )



from typing import Dict

from azureml.core import Datastore, Workspace
from azureml.core.authentication import ServicePrincipalAuthentication
from azureml.core.compute import AmlCompute, ComputeTarget
from azureml.core.environment import Environment
from azureml.exceptions import ComputeTargetException

from azure_helper.logger import get_logger

log = get_logger()


class AMLInterface:
    def __init__(
        self,
        spn_credentials: Dict[str, str],
        subscription_id: str,
        workspace_name: str,
        resource_group: str,
    ):
        """Instantiate an Azure Machine Learning Workspace and its connection to Azure DevOps.

        Parameters
        ----------
        spn_credentials : Dict[str, str]
            Credentials of the Service Principal used to communicate betwee the different resources of the workspace.
            Must contains the TenantID and the ServicePrincipalID.
        subscription_id : str
            The Azure subscription ID containing the workspace.
        workspace_name : str
            The workspace name. The name must be between 2 and 32 characters long.
            The first character of the name must be alphanumeric (letter or number), but the rest of the name may
            contain alphanumerics, hyphens, and underscores. Whitespace is not allowed.
        resource_group : str
            The resource group containing the workspace.
        """
        auth = ServicePrincipalAuthentication(**spn_credentials)
        self.workspace = Workspace(
            workspace_name=workspace_name,
            auth=auth,
            subscription_id=subscription_id,
            resource_group=resource_group,
        )

    def register_datastore(
        self,
        datastore_name: str,
        container_name: str,
        storage_acct_name: str,
        storage_acct_key: str,
    ):
        """Register an Azure Blob Container to the datastore.

        Parameters
        ----------
        datastore_name : str
             The name of the datastore, case insensitive, can only contain alphanumeric characters and _.
        blob_container : str
            The name of the azure blob container.
        storage_acct_name : str
            The storage account name.
        storage_acct_key : str
            Access keys of your storage account, defaults to None.
        """
        Datastore.register_azure_blob_container(
            workspace=self.workspace,
            datastore_name=datastore_name,
            container_name=container_name,
            account_name=storage_acct_name,
            account_key=storage_acct_key,
        )

    def register_aml_environment(self, environment: Environment):
        """Register the environment object in your workspace.

        Parameters
        ----------
        environment : Environment
            A reproducible Python environment for machine learning experiments.
        """
        environment.register(workspace=self.workspace)

    def get_compute_target(self, compute_name: str, vm_size: str = "") -> ComputeTarget:
        """Instantiate a compute instance to train the models.

        Parameters
        ----------
        compute_name : str
            The name of the compute instance.
        vm_size : str, optional
            The size of agent VMs in the the compute instance. More details can be found here: https://aka.ms/azureml-vm-details.
            Note that not all sizes are available in all regions, as detailed in the previous link.
            If not specified, defaults to Standard_NC6. By default ""

        Returns
        -------
        ComputeTarget
            An instantiated compute instance.
        """
        try:
            compute_target = ComputeTarget(
                workspace=self.workspace,
                name=compute_name,
            )
            log.info("Found existing compute target")
            log.info(
                f"Compute target instantiated : {compute_target.status.serialize()}",
            )
        except ComputeTargetException as err:
            log.info(f"No compute target found. Creating a new compute target. {err}")
            compute_config = AmlCompute.provisioning_configuration(
                vm_size=vm_size,
                min_nodes=1,
                max_nodes=2,
            )
            compute_target = ComputeTarget.create(
                self.workspace,
                compute_name,
                compute_config,
            )
            compute_target.wait_for_completion(
                show_output=True,
                timeout_in_minutes=10,
            )
            log.info(
                f"Compute target instantiated : {compute_target.status.serialize()}",
            )
        return compute_target



from unittest.mock import Mock

from azureml.core import ScriptRunConfig

from azure_helper.steps.create_aml_experiment import AMLExperiment

test_module = "azure_helper.steps.create_aml_experiment"


class TestAMLExperiment:
    def test_submit_run(self, mocker):
        # We are not testing Azure ML SDK functionality in a unit test
        # Rather that the correct calls are made

        mock_env = mocker.patch(f"{test_module}.Environment")

        # we mock the Experiment class
        mock_experiment = mocker.patch(f"{test_module}.Experiment")
        # this class is instantiate in AMLExperiment and must return an object when submitted
        # so we mock both : the instantiated class and the returned object when submitted
        mock_experiment_obj = Mock()
        mock_run = Mock()
        mock_experiment.return_value = mock_experiment_obj
        mock_experiment_obj.submit.return_value = mock_run

        mock_aml_inteface = mocker.patch(f"{test_module}.AMLInterface")

        workspace_name = "test_workspace"
        resource_group = "test_rg"
        subscription_id = "test_sub_id"

        spn_credentials = {
            "tenant_id": "test_tenant_id",
            "service_principal_id": "test_spn_id",
            "service_principal_password": "test_spn_passwd",
        }

        aml_interface = mock_aml_inteface(
            spn_credentials,
            subscription_id,
            workspace_name,
            resource_group,
        )

        aml_exp = AMLExperiment(
            aml_interface=aml_interface,
            aml_compute_name="test_compute_name",
            aml_compute_instance="STANDARD_D2_V2",
            env_name="test_env_name",
            experiment_name="test_exp_name",
        )

        aml_exp.submit_run()

        mock_env.get.assert_called_once()

        mock_experiment_obj.submit.assert_called_once()

        aml_interface.get_compute_target.assert_called_once()

        _, kwargs = mock_experiment_obj.submit.call_args_list[0]
        exp_config = kwargs["config"]
        assert isinstance(exp_config, ScriptRunConfig)
        assert exp_config.script == "azure_helper/steps/train.py"

        mock_run.wait_for_completion.assert_called_once_with(show_output=True)
        mock_run.get_metrics.assert_called_once()



# from setuptools import find_packages, setup

# data = dict(
#     name="my_custom_package",
#     version="0.0.3",
#     packages=find_packages(),
# )

# if __name__ == "__main__":
#     setup(**data)



"""
This file is for unit testing the module
my_custom_package.utils.aml_interface

Just as with other similar files, the aim is to test
the correct Azure SDK classes are instantiated and
methods are called, rather than testing the functionality
of the external classes themselves.
"""

from unittest.mock import Mock

from azureml.exceptions import ComputeTargetException
from pytest import fixture

from azure_helper.utils.aml_interface import AMLInterface

test_module = "azure_helper.utils.aml_interface"


@fixture
def aml_interface(mocker):
    mocker_spn = mocker.patch(
        f"{test_module}.ServicePrincipalAuthentication",
    )
    mock_workspace = mocker.patch(
        f"{test_module}.Workspace",
        return_value="test_workspace",
    )
    spn_credentials = {
        "tenant_id": "test_tenant_id",
        "service_principal_id": "test_spn_id",
        "service_principal_password": "test_spn_password",
    }
    return AMLInterface(
        spn_credentials=spn_credentials,
        subscription_id="test_subscription_id",
        workspace_name="test_workspace_name",
        resource_group="test_resource_group",
    )


class TestAMLInterface:
    def test_register_datastore(self, mocker, aml_interface):

        mock_datastore = mocker.patch(
            f"{test_module}.Datastore",
        )

        aml_interface.register_datastore(
            "test_datastore_name",
            "test_blob_container",
            "test_storage_acct_name",
            "test_storage_acct_key",
        )

        mock_datastore.register_azure_blob_container.assert_called_once_with(
            workspace="test_workspace",
            datastore_name="test_datastore_name",
            container_name="test_blob_container",
            account_name="test_storage_acct_name",
            account_key="test_storage_acct_key",
        )

    def register_aml_environment(self, aml_interface):
        mock_environment = Mock()

        aml_interface.register_aml_environment(mock_environment)

        mock_environment.register.assert_called_once_with("test_workspace")

    def test_get_compute_target(self, mocker, aml_interface):
        mock_compute_target = mocker.patch(
            f"{test_module}.ComputeTarget",
        )
        mock_AmlCompute = mocker.patch(
            f"{test_module}.AmlCompute",
        )

        mock_compute_target_obj = Mock()
        mock_compute_target.create.return_value = mock_compute_target_obj

        # First call to mock_compute_target returns 'test_compute_target'
        compute_target = aml_interface.get_compute_target(
            "test_compute_name",
            "STANDARD_D2_V2",
        )
        #  {'currentNodeCount': 1, 'targetNodeCount': 1, 'nodeStateCounts': {'preparingNodeCount': 0, 'runningNodeCount': 0, 'idleNodeCount': 1, 'unusableNodeCount': 0, 'leavingNodeCount': 0, 'preemptedNodeCount': 0}, 'allocationState': 'Steady', 'allocationStateTransitionTime': '2022-08-11T13:48:24.409000+00:00', 'errors': None, 'creationTime': '2022-08-11T13:46:01.432646+00:00', 'modifiedTime': '2022-08-11T13:46:08.602275+00:00', 'provisioningState': 'Succeeded', 'provisioningStateTransitionTime': None, 'scaleSettings': {'minNodeCount': 1, 'maxNodeCount': 2, 'nodeIdleTimeBeforeScaleDown': 'PT1800S'}, 'vmPriority': 'Dedicated', 'vmSize': 'STANDARD_D2_V2'}
        # assert mock_compute_target_obj.status.serialized().vmSize == "STANDARD_D2_V2"

        mock_compute_target.create.assert_not_called()

        mock_compute_target.side_effect = ComputeTargetException(
            "Compute Target Not Found",
        )
        mock_compute = Mock()
        # Compute target exists, create not called
        mock_compute_target.create.return_value = mock_compute

        # Second call to mock_compute_target raises ComputeTargetException
        # Suggesting the compute target needs to be created
        output_2 = aml_interface.get_compute_target(
            "test_compute_name",
            "STANDARD_D2_V2",
        )

        assert output_2 == mock_compute

        mock_AmlCompute.provisioning_configuration.assert_called_once_with(
            vm_size="STANDARD_D2_V2",
            min_nodes=1,
            max_nodes=2,
        )
        mock_compute_target.create.assert_called_once()
        mock_compute.wait_for_completion.assert_called_once()



"""
This file is for unit testing the module
my_custom_package.utils.blob_storage_interface

Just as with other similar files, the aim is to test
the correct Azure SDK classes are instantiated and
methods are called, rather than testing the functionality
of the external classes themselves.
"""

from unittest.mock import Mock

import pandas as pd
from azure.core.exceptions import ResourceExistsError
from pytest import fixture

from azure_helper.utils.blob_storage_interface import BlobStorageInterface

test_module = "azure_helper.utils.blob_storage_interface"


@fixture
def blob_storage_resources(mocker):
    blob_service_client_obj = Mock()

    mock_blob_service_client = mocker.patch(f"{test_module}.BlobServiceClient")
    mock_blob_service_client.from_connection_string.return_value = (
        blob_service_client_obj
    )

    blob_storage_interface = BlobStorageInterface(
        "test_storage_acct_name",
        "test_storage_acct_key",
    )

    return mock_blob_service_client, blob_service_client_obj, blob_storage_interface


class TestBlobStorageInterface:
    def test_init(self, blob_storage_resources):

        mock_blob_service_client, _, _ = blob_storage_resources

        mock_blob_service_client.from_connection_string.assert_called_once_with(
            "DefaultEndpointsProtocol=https;"
            + "AccountName=test_storage_acct_name;"
            + "AccountKey=test_storage_acct_key;"
            + "EndpointSuffix=core.windows.net",
        )

    def test_create_container(self, blob_storage_resources, caplog):

        _, blob_service_client_obj, blob_storage_interface = blob_storage_resources

        blob_service_client_obj.create_container.side_effect = [
            None,
            ResourceExistsError,
        ]
        blob_storage_interface.create_container("test_container_name")

        blob_service_client_obj.create_container.assert_called_with(
            "test_container_name",
        )

        # Upon second call to BlobStorageInterface.create_container,
        # ResourceExistsError is raised from
        # BlobStorageInterface.blob_service_client.create_container
        # Second call checks that no error is raised from
        # BlobStorageInterface.create_container
        # with pytest.raises(ResourceExistsError) as err:
        blob_storage_interface.create_container("test_container_name")
        assert len(caplog.records) == 2
        assert (
            "Blob storage container test_container_name already exists"
            in caplog.records[1].message
        )

    def test_upload_df_to_blob(self, blob_storage_resources, caplog):

        _, blob_service_client_obj, blob_storage_interface = blob_storage_resources

        mock_blob_client = Mock()
        mock_blob_client.upload_blob.side_effect = [
            None,
            ResourceExistsError,
            None,
        ]
        blob_service_client_obj.get_blob_client.return_value = mock_blob_client

        test_df = pd.DataFrame([{"a": 1, "b": 2}, {"a": 3, "b": 4}])

        blob_storage_interface.upload_df_to_blob(
            test_df,
            "test_container_name",
            "test_remote_path",
        )

        assert (
            "Dataset uploaded at blob path : test_remote_path."
            in caplog.records[1].message
        )

        blob_service_client_obj.get_blob_client.assert_called_with(
            container="test_container_name",
            blob="test_remote_path",
        )
        mock_blob_client.upload_blob.assert_called()
        args, _ = mock_blob_client.upload_blob.call_args_list[0]
        upload_arg = args[0]
        # The file uploaded is uploaded as bytes
        assert isinstance(upload_arg, bytes)

        # First time upload_df_to_blob is called, there is no
        # ResourceExistsError so the blob is not deleted
        mock_blob_client.delete_blob.assert_not_called()

        # 2nd call to upload_df_to_blob
        blob_storage_interface.upload_df_to_blob(
            test_df,
            "test_container_name",
            "test_remote_path",
        )
        assert len(caplog.records) == 5
        assert (
            "Blob path test_remote_path already contains datas. Now deleting old datas tu upload the new ones."
            in caplog.records[3].message
        )
        # Second time upload_df_to_blob is called, there is a
        # ResourceExistsError raised so the blob is deleted first
        mock_blob_client.delete_blob.assert_called_once()
        mock_blob_client.upload_blob.assert_called()

    def test_download_blob_to_df(self, blob_storage_resources, caplog):

        _, blob_service_client_obj, blob_storage_interface = blob_storage_resources

        mock_blob_client = Mock()
        mock_stream = Mock()
        test_csv_str = "a,b\n1,2\n3,4"

        blob_service_client_obj.get_blob_client.return_value = mock_blob_client
        mock_blob_client.download_blob.return_value = mock_stream
        mock_stream.content_as_text.return_value = test_csv_str

        output_df = blob_storage_interface.download_blob_to_df(
            "test_container_name",
            "test_remote_path",
        )

        assert isinstance(output_df, pd.DataFrame)
        assert "a" in output_df.columns
        assert output_df.loc[1, "b"] == 4

        blob_service_client_obj.get_blob_client.assert_called_once_with(
            container="test_container_name",
            blob="test_remote_path",
        )
        mock_blob_client.download_blob.assert_called_once()
        mock_stream.content_as_text.assert_called_once()

        assert len(caplog.records) == 1
        assert (
            "Download from test_container_name ended successfully."
            in caplog.records[0].message
        )
